==> entity_mesh-1.1.4.tar.gz: entity_mesh/src/Entity.php <==
<?php

namespace Drupal\entity_mesh;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\StatementInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\TranslatableInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Symfony\Component\Routing\Route;

/**
 * Service description.
 */
abstract class Entity {

  /**
   * The mesh type.
   *
   * @var string
   */
  protected $type;

  /**
   * Entity Mesh Repository.
   *
   * @var \Drupal\entity_mesh\RepositoryInterface
   */
  protected $entityMeshRepository;

  /**
   * The config factory manager.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $config;

  /**
   * The entity repository.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Constructs a Menu object.
   *
   * @param \Drupal\entity_mesh\RepositoryInterface $entity_mesh_repository
   *   The database connection.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity repository.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory manager.
   */
  public function __construct(RepositoryInterface $entity_mesh_repository, EntityTypeManagerInterface $entity_type_manager, LanguageManagerInterface $language_manager, ConfigFactoryInterface $config_factory) {
    $this->entityMeshRepository = $entity_mesh_repository;
    $this->entityTypeManager = $entity_type_manager;
    $this->languageManager = $language_manager;
    $this->config = $config_factory;
  }

  /**
   * Process entity and store source with targets in database.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to process.
   */
  public function processEntity(EntityInterface $entity) {
    // Delete source before process entities.
    $source = $this->createSourceFromEntity($entity);
    if (!$source instanceof SourceInterface) {
      return;
    }
    $this->entityMeshRepository->deleteSource($source);

    // Check if entity is translatable.
    if ($entity instanceof TranslatableInterface && $entity->isTranslatable()) {
      $this->processTranslatableEntity($entity);
      return;
    }
    $this->processEntityItem($entity);
  }

  /**
   * Process a translatable Entity.
   *
   * @param \Drupal\Core\Entity\TranslatableInterface $entity
   *   The entity to process.
   */
  protected function processTranslatableEntity(TranslatableInterface $entity) {

    $translations = $entity->getTranslationLanguages();
    $langcodes = array_keys($translations);
    foreach ($langcodes as $langcode) {

      $translation = $entity->getTranslation($langcode);
      if ($translation instanceof EntityInterface) {
        $this->processEntityItem($translation);
      }
    }
  }

  /**
   * Process an Entity.
   *
   * Generate the source object with its targets and store it in the database.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to process.
   */
  protected function processEntityItem(EntityInterface $entity) {
    $source = $this->createSourceFromEntity($entity);
    if (!$source instanceof SourceInterface) {
      return;
    }
    $this->entityMeshRepository->saveSource($source);
  }

  /**
   * Delete an item from the database.
   *
   * @param string $entity_type
   *   The entity type of the source.
   * @param string $entity_id
   *   The entity id of the source.
   * @param string $type
   *   The type of the source.
   */
  public function deleteItem(string $entity_type, string $entity_id, string $type = '') {
    $source = $this->entityMeshRepository->instanceEmptySource();
    $source->setType($type);
    $source->setSourceEntityType($entity_type);
    $source->setSourceEntityId($entity_id);
    $this->entityMeshRepository->deleteSource($source);
  }

  /**
   * Build a Source object from an Entity, included its targets.
   *
   * Contain all the logic to get all the targets from the entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to process.
   *
   * @return \Drupal\entity_mesh\SourceInterface|null
   *   The source object.
   */
  public function createSourceFromEntity(EntityInterface $entity): ?SourceInterface {
    $source = $this->entityMeshRepository->instanceEmptySource();
    $source->setType($this->type);
    $source->setSourceEntityType($entity->getEntityTypeId());
    $source->setSourceEntityBundle($entity->bundle());
    $source->setSourceEntityId((string) $entity->id());
    $source->setSourceEntityLangcode($entity->language()->getId());
    return $source;
  }

  /**
   * Get the front page entity.
   */
  public function frontPage() {
    $config = $this->config->get('system.site');
    $front = $config->get('page.front');
    $query = $this->entityMeshRepository->getDatabaseService()->select('path_alias', 'pa');
    $query->fields('pa', ['path']);
    $or = $query->orConditionGroup()
      ->condition('alias', $front)
      ->condition('path', $front);
    $query->condition($or);
    // @todo !
    // $query->condition('langcode', $langcode);
    $result = $query->execute();
    if (!$result instanceof StatementInterface) {
      return;
    }
    $record = $result->fetchObject();
    if ($record) {
      $path = explode('/', ltrim($record->path, '/'));
      $storage = $this->entityTypeManager->getStorage($path[0]);
      $entity = $storage->load($path[1]);
      if ($entity instanceof EntityInterface) {
        return $entity;
      }
    }
  }

  /**
   * Get the Entity Mesh configuration.
   */
  protected function getMeshConfiguration() {
    return $this->config->get('entity_mesh.settings');
  }

  /**
   * Check if the internal target should be processed.
   */
  protected function ifProcessInternalTarget() {
    return $this->config->get('entity_mesh.settings')->get('internal') ?? FALSE;
  }

  /**
   * Check if the external target should be processed.
   */
  protected function ifProcessExternalTarget() {
    return $this->config->get('entity_mesh.settings')->get('external') ?? FALSE;
  }

  /**
   * Check if the route match belongs to a entity route.
   *
   * @param array $route_match
   *   The route match.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The entity or NULL.
   */
  protected function checkAndGetEntityFromEntityRoute(array $route_match):? EntityInterface {
    $entity = NULL;

    // Check that we have the parameters that we need.
    if (!isset($route_match['_route']) ||
      !isset($route_match['_route_object']) ||
      !$route_match['_route_object'] instanceof Route
    ) {
      return $entity;
    }

    // Check if the route start with the string 'entity.'.
    // If it is the case, it is probably an entity route.
    if (strpos($route_match['_route'], 'entity.') !== 0) {
      return $entity;
    }

    // If it is an entity route, we can get from parameters of the Route object
    // the name of the parameter.
    /** @var \Symfony\Component\Routing\Route $route_object */
    $route_object = $route_match['_route_object'];
    $parameters = $route_object->getOption('parameters');

    foreach ($parameters as $name => $value) {
      // There has to be a converter and it has to be the entity converter.
      if (isset($value['converter']) && $value['converter'] === 'paramconverter.entity') {
        $entity = (isset($route_match[$name]) && $route_match[$name] instanceof EntityInterface) ? $route_match[$name] : NULL;
        break;
      }
    }

    return $entity;
  }

}

==> entity_mesh-1.1.6.tar.gz: entity_mesh/src/Entity.php <==
<?php

namespace Drupal\entity_mesh;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\StatementInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\TranslatableInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Symfony\Component\Routing\Route;

/**
 * Service description.
 */
abstract class Entity {

  /**
   * The mesh type.
   *
   * @var string
   */
  protected $type;

  /**
   * Entity Mesh Repository.
   *
   * @var \Drupal\entity_mesh\RepositoryInterface
   */
  protected $entityMeshRepository;

  /**
   * The config factory manager.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $config;

  /**
   * The entity repository.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Constructs a Menu object.
   *
   * @param \Drupal\entity_mesh\RepositoryInterface $entity_mesh_repository
   *   The database connection.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity repository.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory manager.
   */
  public function __construct(RepositoryInterface $entity_mesh_repository, EntityTypeManagerInterface $entity_type_manager, LanguageManagerInterface $language_manager, ConfigFactoryInterface $config_factory) {
    $this->entityMeshRepository = $entity_mesh_repository;
    $this->entityTypeManager = $entity_type_manager;
    $this->languageManager = $language_manager;
    $this->config = $config_factory;
  }

  /**
   * Process entity and store source with targets in database.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to process.
   */
  public function processEntity(EntityInterface $entity) {
    // Delete source before process entities.
    $source = $this->createSourceFromEntity($entity);
    if (!$source instanceof SourceInterface) {
      return;
    }
    $this->entityMeshRepository->deleteSource($source);

    // Check if entity is translatable.
    if ($entity instanceof TranslatableInterface && $entity->isTranslatable()) {
      $this->processTranslatableEntity($entity);
      return;
    }
    $this->processEntityItem($entity);
  }

  /**
   * Process a translatable Entity.
   *
   * @param \Drupal\Core\Entity\TranslatableInterface $entity
   *   The entity to process.
   */
  protected function processTranslatableEntity(TranslatableInterface $entity) {

    $translations = $entity->getTranslationLanguages();
    $langcodes = array_keys($translations);
    // Check access with anonymous user to the translation.
    $user = $this->getMeshUser();
    foreach ($langcodes as $langcode) {

      $translation = $entity->getTranslation($langcode);
      if ($translation instanceof EntityInterface && $translation->access('view', $user)) {
        $this->processEntityItem($translation);
      }
    }
  }

  /**
   * Get the current user.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The user entity or NULL.
   */
  protected function getMeshUser() {
    $user = $this->entityTypeManager->getStorage('user')->load(0);
    if ($user instanceof EntityInterface) {
      return $user;
    }
    return NULL;

  }

  /**
   * Process an Entity.
   *
   * Generate the source object with its targets and store it in the database.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to process.
   */
  protected function processEntityItem(EntityInterface $entity) {
    $source = $this->createSourceFromEntity($entity);
    if (!$source instanceof SourceInterface) {
      return;
    }
    $this->entityMeshRepository->saveSource($source);
  }

  /**
   * Delete an item from the database.
   *
   * @param string $entity_type
   *   The entity type of the source.
   * @param string $entity_id
   *   The entity id of the source.
   * @param string $type
   *   The type of the source.
   */
  public function deleteItem(string $entity_type, string $entity_id, string $type = '') {
    $source = $this->entityMeshRepository->instanceEmptySource();
    $source->setType($type);
    $source->setSourceEntityType($entity_type);
    $source->setSourceEntityId($entity_id);
    $this->entityMeshRepository->deleteSource($source);
  }

  /**
   * Build a Source object from an Entity, included its targets.
   *
   * Contain all the logic to get all the targets from the entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to process.
   *
   * @return \Drupal\entity_mesh\SourceInterface|null
   *   The source object.
   */
  public function createSourceFromEntity(EntityInterface $entity): ?SourceInterface {
    $source = $this->entityMeshRepository->instanceEmptySource();
    $source->setType($this->type);
    $source->setSourceEntityType($entity->getEntityTypeId());
    $source->setSourceEntityBundle($entity->bundle());
    $source->setSourceEntityId((string) $entity->id());
    $source->setSourceEntityLangcode($entity->language()->getId());
    return $source;
  }

  /**
   * Get the front page entity.
   */
  public function frontPage() {
    $config = $this->config->get('system.site');
    $front = $config->get('page.front');
    $query = $this->entityMeshRepository->getDatabaseService()->select('path_alias', 'pa');
    $query->fields('pa', ['path']);
    $or = $query->orConditionGroup()
      ->condition('alias', $front)
      ->condition('path', $front);
    $query->condition($or);
    // @todo !
    // $query->condition('langcode', $langcode);
    $result = $query->execute();
    if (!$result instanceof StatementInterface) {
      return;
    }
    $record = $result->fetchObject();
    if ($record) {
      $path = explode('/', ltrim($record->path, '/'));
      $storage = $this->entityTypeManager->getStorage($path[0]);
      $entity = $storage->load($path[1]);
      if ($entity instanceof EntityInterface) {
        return $entity;
      }
    }
  }

  /**
   * Get the Entity Mesh configuration.
   */
  protected function getMeshConfiguration() {
    return $this->config->get('entity_mesh.settings');
  }

  /**
   * Check if the internal target should be processed.
   */
  protected function ifProcessInternalTarget() {
    return $this->config->get('entity_mesh.settings')->get('internal') ?? FALSE;
  }

  /**
   * Check if the external target should be processed.
   */
  protected function ifProcessExternalTarget() {
    return $this->config->get('entity_mesh.settings')->get('external') ?? FALSE;
  }

  /**
   * Check if the route match belongs to a entity route.
   *
   * @param array $route_match
   *   The route match.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The entity or NULL.
   */
  protected function checkAndGetEntityFromEntityRoute(array $route_match):? EntityInterface {
    $entity = NULL;

    // Check that we have the parameters that we need.
    if (!isset($route_match['_route']) ||
      !isset($route_match['_route_object']) ||
      !$route_match['_route_object'] instanceof Route
    ) {
      return $entity;
    }

    // Check if the route start with the string 'entity.'.
    // If it is the case, it is probably an entity route.
    if (strpos($route_match['_route'], 'entity.') !== 0) {
      return $entity;
    }

    // If it is an entity route, we can get from parameters of the Route object
    // the name of the parameter.
    /** @var \Symfony\Component\Routing\Route $route_object */
    $route_object = $route_match['_route_object'];
    $parameters = $route_object->getOption('parameters');

    foreach ($parameters as $name => $value) {
      // There has to be a converter and it has to be the entity converter.
      if (isset($value['converter']) && $value['converter'] === 'paramconverter.entity') {
        $entity = (isset($route_match[$name]) && $route_match[$name] instanceof EntityInterface) ? $route_match[$name] : NULL;
        break;
      }
    }

    return $entity;
  }

}
